# -*- coding: utf-8 -*-
"""Setup the userprofile application"""

from userprofile import model
from tgext.pluggable import app_model

def bootstrap(command, conf, vars):
    print('Bootstrapping userprofile...')



from ming import schema as s
from ming.odm import FieldProperty
from ming.odm.declarative import MappedClass
from datetime import datetime

from userprofile.model import DBSession
from tgext.pluggable import app_model
from tg import url
from tg.decorators import cached_property
from tgext.pluggable.utils import mount_point


class ProfileActivation(MappedClass):
    class __mongometa__:
        session = DBSession
        name = 'userprofile_activation'

    _id = FieldProperty(s.ObjectId)
    activated = FieldProperty(s.DateTime)

    old_email_address = FieldProperty(s.String, required=True)
    email_address = FieldProperty(s.String, required=True)

    activation_code = FieldProperty(s.String, required=True, unique=True)

    @cached_property
    def activation_link(self):
        return url(mount_point('userprofile') + '/activate',
                   params=dict(activation_code=self.activation_code),
                   qualified=True)

    @classmethod
    def generate_activation_code(cls, email):
        from hashlib import sha1
        import hmac
        return hmac.new(str(email), str(datetime.now()), sha1).hexdigest()

    @classmethod
    def by_code(cls, code):
        return cls.query.find({'activated': None, 'activation_code': code}).first()



# -*- coding: utf-8 -*-
from hashlib import md5
import tg
from tg import url, request, config
from tgext.pluggable import app_model, plug_url
from tg.i18n import ugettext as _, lazy_ugettext as l_

from tw2.core import Required, MatchValidator
from tw2.forms import ListForm, TextField, TextArea, HiddenField,\
    FileField, SubmitButton, PasswordField, FileValidator

from tgext.mailer import Message as message
from tgext.mailer import get_mailer


def send_email(to_addr, sender, subject, body, rich=None):
    mailer = get_mailer(request)
    message_to_send = message(
        subject=subject,
        sender=sender,
        recipients=[to_addr],
        body=body,
        html=rich or None,
    )
    if config.get('tm.enabled', False):
        mailer.send(message_to_send)
    else:
        mailer.send_immediately(message_to_send)


def get_profile_css(config):
    return url(config['_pluggable_userprofile_config'].get(
        'custom_css', '/_pluggable/userprofile/css/style.css'))


def _get_user_gravatar(email_address):
    if not isinstance(email_address, bytes):
        email_address = email_address.encode('utf-8')
    mhash = md5(email_address).hexdigest()
    return url('http://www.gravatar.com/avatar/'+mhash, params=dict(s=32))


def get_user_data(user):
    user_data = getattr(user, 'profile_data', {
        'display_name': (l_('Display Name'), user.display_name),
        'email_address': (l_('Email Address'), user.email_address),
    })

    user_avatar = user_data.pop('avatar', [None, None])[1]

    if user_avatar is None:
        fbauth_info = getattr(user, 'fbauth', None)
        if fbauth_info is not None:
            user_avatar = fbauth_info.profile_picture + '?type=large'
        else:
            user_avatar = _get_user_gravatar(user_data['email_address'][1])

    return user_data, user_avatar


def update_user_data(user, user_data):
    for k, v in user_data.items():
        setattr(user, k, v)


class ImageField(FileField):
    inline_engine_name = 'kajiki'
    template = '''
    <div py:strip="True">
        <script>
            function filefieldPreview(event, id) {
                var output = document.getElementById(id);
                output.style.backgroundImage = "url('" + URL.createObjectURL(event.target.files[0]) + "')";
            }
        </script>
        <div id="${w.attrs['id']}-imagefield">
            <div id="${w.attrs['id']}-image" class="userprofile-imagefield-image"
                 style="background-image:url('${w.initial_src}');">
                <label for="${w.attrs['id']}">
                    <span class="glyphicon glyphicon-pencil" aria-hidden="true"> </span>
                     <span id="${w.attrs['id']}-edit-text"> &nbsp; Edit</span>

                </label>

            </div>
            <input style="display: none;" py:attrs="w.attrs"
                   onchange="filefieldPreview(event, '${w.attrs['id']}-image')" accept="image/*"/>
       </div>
    </div>
    '''


class ImageValidator(FileValidator):
    extension = ('png', 'jpeg', 'jpg')


password_not_prefilled = {
    'autocomplete': 'new-password',  # currentely works only with chrome
    'readonly': '',
    'onfocus': """if (this.hasAttribute('readonly')) {
        this.removeAttribute('readonly');
    }"""
}


class UserForm(ListForm):
    nothing = HiddenField()  # just to create children
    submit = SubmitButton(value=l_('Save'))


def create_user_form(user):
    profile_form = getattr(user, 'profile_form', None)
    if not profile_form:
        user_data, user_avatar = get_user_data(user)
        profile_form = UserForm()
        profile_form.child = profile_form.child()

        profile_form.child.children.append(
            ImageField(id='avatar', name="avatar", initial_src=user_avatar))

        for name, info in user_data.items():
            profile_form.child.children.append(
                TextField(id=name, validator=Required, label=info[0]))

        profile_form.child.children.append(
            PasswordField(id='password', label=u'New Password', attrs=password_not_prefilled))
        profile_form.child.children.append(
            PasswordField(id='verify_password', label=l_(u'Confirm New Password')))
        profile_form.child.validator = FieldsMatch('password', 'verify_password')

        profile_form = profile_form()
    return profile_form



# -*- coding: utf-8 -*-
"""The userprofile package"""
from tg.configuration import milestones
from tgext.pluggable import plug
from tgext.pluggable import plugged

from userprofile import model


def plugme(app_config, options):
    app_config['_pluggable_userprofile_config'] = options
    milestones.config_ready.register(model.configure_models)

    if 'resetpassword' not in plugged(config=app_config):
        plug(app_config, 'resetpassword')

    return dict(appid='userprofile', global_helpers=False)



# -*- coding: utf-8 -*-
"""Main Controller"""
from datetime import datetime

from resetpassword.lib import generate_token
from tg import TGController, expose, flash, require, predicates, \
    request, redirect, config, override_template, abort
from tg.i18n import ugettext as _, lazy_ugettext as l_
from tg.predicates import not_anonymous
from tg.util import Bunch
from tgext.pluggable import plug_url, app_model
from tw2.core import ValidationError

from userprofile import model
from userprofile.lib import create_user_form, get_user_data, get_profile_css, \
                            update_user_data, send_email


class RootController(TGController):
    @expose('userprofile.templates.index')
    @require(predicates.not_anonymous())
    def index(self):
        user = request.identity['user']
        user_data, user_avatar = get_user_data(user)
        user_displayname = user_data.pop('display_name', (None, 'Unknown'))
        user_partial = config['_pluggable_userprofile_config'].get('user_partial')
        return dict(user=user.profile_data,
                    user_data=user_data,
                    user_avatar=user_avatar,
                    user_displayname=user_displayname,
                    profile_css=get_profile_css(config),
                    user_partial=user_partial)

    @expose('userprofile.templates.edit')
    @require(predicates.not_anonymous())
    def edit(self):
        user = request.identity['user']
        user_data, user_avatar = get_user_data(user)
        user_data = Bunch(((fieldid, info[1]) for fieldid, info in user_data.items()))
        return dict(user=user, profile_css=get_profile_css(config),
                    user_avatar=user_avatar,
                    form=create_user_form(user))

    @expose()
    @require(predicates.not_anonymous())
    def save(self, **kw):
        kw.pop('nothing')
        flash_message = _('Profile successfully updated')
        user = request.identity['user']

        # validate the form
        try:
            form = create_user_form(user)
            kw = form.validate(kw)
        except ValidationError:
            override_template(self.save, 'kajiki:userprofile.templates.edit')
            user_data, user_avatar = get_user_data(user)
            return dict(user={},
                        profile_css=get_profile_css(config),
                        user_avatar=user_avatar,
                        form=form)

        # get the profile_save function that may be custom
        profile_save = getattr(user, 'save_profile', None)
        if not profile_save:
            profile_save = update_user_data

        # do not change the password if the user did't set it

        if 'password' in kw:
            if not kw.get('password'):  # keep old email
                kw.pop('password')
                kw.pop('verify_password')
            else:  # set new email
                kw.pop('verify_password')

        # we don't want to save the email until it is confirmed
        new_email = kw['email_address']
        kw['email_address'] = user.email_address
        profile_save(user, kw)

        if new_email != user.email_address:
            # save this new email in the db
            dictionary = {
                'old_email_address': user.email_address,
                'email_address': new_email,
                'activation_code':
                    model.ProfileActivation.generate_activation_code(new_email),
            }
            activation = model.provider.create(model.ProfileActivation, dictionary)

            # ok, send the email please
            userprofile_config = config.get('_pluggable_userprofile_config')
            mail_body = userprofile_config.get(
                'mail_body',
                _('Please click on this link to confermate your email address')
                + '\n\n' + activation.activation_link,
            )
            email_data = {'sender': config['userprofile.email_sender'],
                          'subject': userprofile_config.get(
                              'mail_subject', _('Please confirm your email')),
                          'body': mail_body,
                          'rich': userprofile_config.get('mail_rich', '')}
            send_email(new_email, **email_data)
            flash_message += '.\n' + _('Confirm your email please')

        flash(flash_message)
        return redirect(plug_url('userprofile', '/'))

    @expose()
    def activate(self, activation_code, **kw):
        activation = model.ProfileActivation.by_code(activation_code) or abort(404)
        user = model.provider.get_obj(app_model.User,
                                      {'email_address': activation.old_email_address})
        user.email_address = activation.email_address
        activation.activated = datetime.utcnow()
        flash(_('email correctely updated'))
        return redirect(plug_url('userprofile', '/'))

    @expose()
    @require(not_anonymous(msg=l_("User must be authenticated")))
    def reset_password(self, redirect_to='/'):
        user = request.identity['user']
        token = generate_token(user, redirect_to=redirect_to)
        return redirect(plug_url('resetpassword', '/change_password/', params=dict(data=token)))



# -*- coding: utf-8 -*-
from tgext.pluggable import PluggableSession
import logging
import tg


log = logging.getLogger(__name__)

DBSession = PluggableSession()
provider = None

ProfileActivation = None


def init_model(app_session):
    DBSession.configure(app_session)


def configure_models():
    global provider, ProfileActivation

    if tg.config.get('use_sqlalchemy', False):
        log.info('Configuring tgapp-userprofile for SQLAlchemy')
        from userprofile.model.sqla.models import ProfileActivation
        from sprox.sa.provider import SAORMProvider
        provider = SAORMProvider(session=DBSession, engine=False)
    elif tg.config.get('use_ming', False):
        log.info('Configuring tgapp-userprofile for Ming')
        from userprofile.model.ming.models import ProfileActivation
        from sprox.mg.provider import MingProvider
        provider = MingProvider(DBSession)
    else:
        raise ValueError('tgapp-userprofile should be used with sqlalchemy or ming')
